#!/bin/bash

MKF="Makefile"

# Truncate makefile
:> $MKF

# Build setup
source ../common/common.def
#SRC_DIR=../../mkxp-freebird
#DEP_DIR=deps

# Arch specific stuff
source arch.def

source ${SRC_DIR}/sources.def

AR=(
  $LIBRUBY_A
  libSDL2main.a
  libSDL2_image.a
  libSDL_sound.a
  libSDL2_ttf.a
#  libfreetype.a
#  libpng16.a
#  libjpeg.a
  libvorbisfile.a
  libvorbis.a
  libogg.a
#  libz.a
  libphysfs.a
#  libpixman-1.a
#  libsigc-2.0.a
  libSDL2.a
  libOpenAL32.a
#  libwebp.a
  libmodplug.a
)

MINGW_AR=(
  $LIBBOOST_A
  libiconv.a
  libfreetype.a
  libbz2.a  # Fedora provided freetype wants this *sigh*
  libpng16.a
  libjpeg.a
  libz.a
  libpixman-1.a
  libsigc-2.0.a
)

AR_FILES=()
for ar in ${AR[*]}
do
	AR_FILES+=("${DEP_DIR}/lib/${ar}")
done

for ar in ${MINGW_AR[*]}
do
	AR_FILES+=("/usr/${MINGW_ARCH}/sys-root/mingw/lib/${ar}")
done

BOOST_I=/usr/${MINGW_ARCH}/sys-root/mingw/include

PKGS=(sigc++-2.0 pixman-1 zlib physfs vorbisfile
      sdl2 SDL2_image SDL2_ttf SDL_sound openal)

source ${DEP_DIR}/buildenv.def
export PKG_CONFIG_PATH

CC=${MINGW_ARCH}-gcc
CXX=${MINGW_ARCH}-g++
STRIP=${MINGW_ARCH}-strip
OBJCOPY=${MINGW_ARCH}-objcopy
WINDRES=${MINGW_ARCH}-windres

CFLAGS=$(pkg-config --cflags ${PKGS[*]})
CFLAGS+=" -pipe -O2 ${ARCH_DEFINES}"

CXXFLAGS="${CFLAGS}"
CXXFLAGS+=" -I${BOOST_I}"
CXXFLAGS+=" -std=c++11"

RUBY_CFLAGS=$(pkg-config --cflags ruby-2.1)


OPENAL_LDFLAGS=$(pkg-config --libs openal)
LDFLAGS="-std=c++11 -lwinmm -lshell32 -lws2_32 -liphlpapi -limagehlp -lshlwapi -Wl,--no-undefined -lm -ldinput8 -ldxguid -ldxerr8 -luser32 -lgdi32 -lwinmm -limm32 -lole32 -loleaut32 -lshell32 -lhid -lsetupapi -lversion -luuid -static -lpthread -lssp"

OBJECTS=()

#if [ $DEBUG == true ]; then
	CXXFLAGS+=" -g"
#fi

echo "Writing embed generation rules"
BUILD_DIR=`pwd`
for embed in ${EMBED[*]}
do
	file="${embed}"
	base=$(basename $file)
	outbase="${base}.xxd"
	out="${BUILD_DIR}/${outbase}"

	echo -ne "${outbase}: ${SRC_DIR}/${file}\n\t" >> $MKF
	echo -e "cd ${SRC_DIR}; xxd -i ${file} > \"${out}\"\n" >> $MKF
	eval "cd ${SRC_DIR}; xxd -i ${file} > \"${out}\"; cd ->/dev/null"
done

function compile_common () {
	file="${SRC_DIR}/${1}"
	base=$(basename --suffix=.cpp $1)
	out="${base}.o"
	flags="-I. -I${SRC_DIR} -I${SRC_DIR}/src -I${SRC_DIR}/windows ${CXXFLAGS}"
	OBJECTS+=" ${out}"

	$CXX -MM $file $flags >> $MKF
	echo -e "\t${CXX} -c -o ${out} ${file} ${flags}\n" >> $MKF
}

# Just for you, steamshim..
function compile_common_c () {
	file="${SRC_DIR}/${1}"
	base=$(basename --suffix=.c $1)
	out="${base}.o"
	flags="-I. -I${SRC_DIR}/src -I${SRC_DIR}/windows ${CFLAGS}"
	OBJECTS+=" ${out}"

	$CC -MM $file $flags >> $MKF
	echo -e "\t${CC} -c -o ${out} ${file} ${flags}\n" >> $MKF
}

echo "Writing src compilation rules"
for src in ${SOURCES[*]}
do
	compile_common "${src}"
done

for src in ${SOURCES_C[*]}
do
	compile_common_c "${src}"
done

function compile_mri () {
	file="${SRC_DIR}/${1}"
	base=$(basename --suffix=.cpp $1)
	out="${base}.o"
	flags="-I. -I${SRC_DIR} -I${SRC_DIR}/src -I${SRC_DIR}/binding-mri ${CXXFLAGS} ${RUBY_CFLAGS}"
	OBJECTS+=" ${out}"

	$CXX -MM $file $flags >> $MKF
	echo -e "\t${CXX} -c -o ${out} ${file} ${flags}\n" >> $MKF
}

echo "Writing binding compilation rules"
for src in ${MRI_SOURCES[*]}
do
	compile_mri "${src}"
done

# Windows resources
echo "Writing winres compilation rules"
RES_O="win-resource.o"
RES_RC="${SRC_DIR}/windows/resource.rc"
echo -n "${RES_O}: " >> $MKF
eval "echo ${SRC_DIR}/windows/*" >> $MKF
echo -e "\t${WINDRES} ${RES_RC} -O coff -o ${RES_O}\n" >> $MKF
OBJECTS+=" ${RES_O}"

EXES=()
function link_exe {
	exe_f_fat="${1}-unstripped.exe"
	echo "${exe_f_fat}: ${OBJECTS[*]}" >> $MKF
	echo -e "\t${CXX} -o ${exe_f_fat} ${OBJECTS[*]} -lmingw32 -static-libgcc -static-libstdc++ ${AR_FILES[*]} ${LDFLAGS} ${2}\n" >> $MKF

	exe_f="${1}.exe"
	dbg_f="${1}.dbg"
	echo "${exe_f} ${dbg_f} &: ${exe_f_fat}" >> $MKF
	echo -e "\t$OBJCOPY --only-keep-debug ${exe_f_fat} ${dbg_f}" >> $MKF
	echo -e "\t${STRIP} -s ${exe_f_fat} -o ${exe_f}\n" >> $MKF
	EXES+=" ${exe_f}"
}

echo "Writing executable linking rules"
link_exe mkxp -mwindows
link_exe mkxp-console

echo -e "all: ${EXES[*]}\n" >> $MKF

echo "clean:" >> $MKF
echo -e "\trm -f *.xxd *.o *.exe *.dbg\n" >> $MKF

echo ".PHONY: all clean" >> $MKF
echo ".DEFAULT_GOAL := all" >> $MKF
